//
//  KSVPNUFacade.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

/*!
 @header
 @abstract Header file
 */

#import <Foundation/Foundation.h>

#import <VPNUnlimitedSDK/VPNUAuthDelegate.h>
#import <VPNUnlimitedSDK/VPNUTokenProvider.h>
#import <VPNUnlimitedSDK/VPNUManager.h>


@class VPNUAccountUserInfo;
@class VPNUPersonalServer;
@class VPNUAccountDevice;
@class VPNUAccountStatus;
@class VPNUCrossAuthData;
@class VPNULocationInfo;
@class VPNUConnection;
@class VPNUServerItem;
@class VPNUManager;

extern NSString * const kVPNUnlimitedSDKVersion;

/*!
 @class KSVPNUFacade
 @abstract
 Class which serves as a 'facade' to other VPN Unlimited SDK classes.
 @discussion
 This class hides the complexity of VPN Unlimited SDK by encapsulating most of the needed features.
 */
@interface KSVPNUFacade : NSObject

/*! @brief Singleton instance of Facade. */
@property (class, readonly) KSVPNUFacade *defaultFacade;

/*!
 @brief Intance of @link VPNUConnection @/link which is used to configure and control VPN connections.
 @discussion This object is read-only and created lazily inside @link KSVPNUFacade@/link implementation.
 @see VPNUConnection
 */
@property (readonly) VPNUConnection *connection;

@property (readonly) VPNUManager *manager;

/*!
 @brief Info about currently logged-in user.
 @see VPNUAccountUserInfo
 */
@property (strong) VPNUAccountUserInfo *currentUserInfo;
/*!
 @brief Info about currently logged-in user's device.
 @see VPNUAccountDevice
 */
@property (strong) VPNUAccountDevice *currentDeviceInfo;
/*!
 @brief Currently logged-in user's account status.
 @see VPNUAccountStatus
 */
@property (strong) VPNUAccountStatus *currentAccountStatus;
/*!
 @brief Currently user's servers list from the @link serverListWithOptions:completion:@/link method.
 @see VPNUServerItem
 */
@property (strong) NSArray<VPNUServerItem *> *currentServersList;


/*!
 @brief Creates all settings for VPN Unlimited SDK (macOS/iOS).
 @discussion .
 @param appID The application ID needed for request signing.
 @param appSecret The application secret key needed for request signing.
 @param applicationGroupIdentifier The shared application group id needed for storing logs.
 @param appVersion The application version.
 @param sharedKeychainGroupID The shared keychain group ID needed for deviceID storing in iOS keychain.
 @param providerBundleIdentifier The provider bundle identifier needed for Network Extension. Needed for iOS and macOS.
 @param wgProviderBundleIdentifier The provider bundle identifier needed for Network Extension (Wireguard protocol). Needed for iOS and macOS.
 @param profileName Custom name for a downloaded VPN profile.
 @param completion Block to be called upon operation completion.
 */
- (void)initializeSettingsWithAppID:(NSString *)appID
                          appSecret:(NSString *)appSecret
         applicationGroupIdentifier:(NSString *)applicationGroupIdentifier
                         appVersion:(NSString *)appVersion
              sharedKeychainGroupID:(NSString *)sharedKeychainGroupID
           providerBundleIdentifier:(NSString *)providerBundleIdentifier
         wgProviderBundleIdentifier:(NSString *)wgProviderBundleIdentifier
                        profileName:(NSString *)profileName
                         completion:(dispatch_block_t)completion;

/*!
 @brief Creates all settings for VPN Unlimited SDK (macOS Standalone).
 @discussion .
 @param appID The application id needed for request signing.
 @param appSecret The application secret key needed for request signing.
 @param applicationGroupIdentifier The shared application group id needed for storing logs.
 @param appVersion The application version. The default value is 1.0.
 @param sharedKeychainGroupID The shared keychain group id needed for deviceID storing in iOS keychain.
 @param helperToolBundleIdentifier The helper tool bundle id needed for install vpn service.
 @param daemonBundleName The VPN service bundle id.
 @param completion Block to be called upon operation completion.
 */
- (void)initializeSettingsOfStandaloneWithAppID:(NSString *)appID
                                      appSecret:(NSString *)appSecret
                     applicationGroupIdentifier:(NSString *)applicationGroupIdentifier
                                     appVersion:(NSString *)appVersion
                          sharedKeychainGroupID:(NSString *)sharedKeychainGroupID
                     helperToolBundleIdentifier:(NSString *)helperToolBundleIdentifier
                               daemonBundleName:(NSString *)daemonBundleName
                                     completion:(dispatch_block_t)completion;


/*!
 @brief A delegate object which conforms to @link //apple_ref/occ/intf/VPNUAuthDelegate@/link protocol.
 @discussion Once you define an object which conforms to @link //apple_ref/occ/intf/VPNUAuthDelegate@/link, set this property with it. If you are using non-standard authorization methods (like authorizing with social networks), you must provide an object which resolves custom authorization with third-party server and returns authentication details.
 */
@property (strong) id<VPNUAuthDelegate> customAuthDelegate;

@property (strong) id<VPNUTokenProvider> tokenProvider;

/*!
 @brief Cancels all pending requests.
 */
- (void)cancelAllRequests;


@end


/*!
 * @typedef KSVPNUServerListOptions
 * @brief Server list fetching options
 */
typedef enum {
    /*! @const KSVPNUServerListOptionsNonAuthorized 0 - authorization is required; 1 - authorization is not required */
    KSVPNUServerListOptionsNonAuthorized = 1 << 0,
    /*! @const KSVPNUServerListOptionsOptimal 0 - default server list will be returned; 1 - list of recomended servers will be returned */
    KSVPNUServerListOptionsOptimal = 1 << 1,
    /*! @const KSVPNUServerListOptionsSquareFlags 0 - round region flag images; 1 - square region flag images */
    KSVPNUServerListOptionsSquareFlags  = 1 << 2,
} KSVPNUServerListOptions;


/*!
 @category Networking
 */

@interface KSVPNUFacade (Networking)

/*!
 @brief Retrieves a user current location with no need to logged-in.
 @param squareFlags YES - square; NO - round
 @param completion Block to be called upon operation completion.
 */
- (void)geolocateMeWithSquareFlags:(BOOL)squareFlags
                        completion:(void(^)(VPNULocationInfo *locationInfo, NSError *error))completion;
/*!
 @brief Retrieves a list of available VPN servers for provided options.
 @param mask Bitmask of @link KSVPNUServerListOptions@/link values. Default is 0.
 @param completion Block to be called upon operation completion.
 @see KSVPNUServerListOptions
 */
- (void)serverListWithOptions:(KSVPNUServerListOptions)mask completion:(void(^)(NSArray<VPNUServerItem *> *serversList, NSError *error))completion;

/*!
 @brief Validates purchases on server.
 @param countryCode Code for the country of purchase.
 @param completion Block to be called upon operation completion.
 */
- (void)validatePurchaseWithCountryCode:(NSString *)countryCode
                             completion:(void(^)(NSError *error))completion;

/*!
 @brief Sends redeem code to server.
 @discussion This method does not perform code validation locally. If you want code validation feature you have to implement it.
 @param code Reddem code string.
 @param completion Block to be called upon operation completion.
 Possible error responses: 351, 352, 502, 503
 @see VPNUAccountUserInfo
 */
- (void)enterCode:(NSString *)code completion:(void(^)(NSError *error))completion;
/*!
 @brief Retrieves the currently logged-in user's device information.
 @param completion Block to be called upon operation completion.
 @see VPNUAccountDevice
 */
- (void)userDeviceInfoWithCompletion:(void(^)(VPNUAccountDevice *deviceInfo, NSError *error))completion;
/*!
 @brief Retrieves the currently logged-in user's account information.
 @param completion Block to be called upon operation completion.
 Possible error responses: 502, 503
 @see VPNUAccountUserInfo
 @see VPNUResponseCode
 */
- (void)userInfoWithCompletion:(void(^)(VPNUAccountUserInfo *userInfo, NSError *error))completion;
/*!
 @brief Retrieves the currently logged-in user's account status.
 @param completion Block to be called upon operation completion.
 @see VPNUAccountStatus
 */
- (void)accountStatusWithCompletion:(void(^)(VPNUAccountStatus *accountStatus, NSError *error))completion;
/*!
 @brief Changes password for a user identifiied by login parameter.
 @discussion If password was changed successfully it automatically reauthorizes user with new password.
 This method doesn't perform any local checks on provided passwords. If you need these checks you have to implement them.
 @param login User's login/email.
 @param oldPassword User's current password.
 @param newPassword User's new password.
 @param completion Block to be called upon operation completion.
 Possible error responses: 302, 323, 500, 502, 503
 @see VPNUResponseCode
 @see authorizeWithLogin:password:completion:
 */
- (void)changeAccountPasswordWithLogin:(NSString *)login
                           oldPassword:(NSString *)oldPassword
                           newPassword:(NSString *)newPassword
                            completion:(void(^)(NSError *error))completion;
/*!
 @brief Performs authorization.
 @discussion This method tries to authorize a user with specified login and password. If authorization is successful it automatically sets the @link currentUserInfo@/link property.
 @param login User's login. Cannot be nil.
 @param password User's password. Cannot be nil.
 @param completion Block to be called upon operation completion.
 */
- (void)authorizeWithLogin:(NSString *)login
                  password:(NSString *)password
                completion:(void(^)(NSError *error))completion;

/*!
 @brief Sends account confirmation email.
 @discussion This method sends an account confirmation letter to user's email.
 @param login User's login.
 @param completion Block to be called upon operation completion.
 Possible error responses: 361, 502, 504
 @see VPNUResponseCode
 */
- (void)sendConfirmationEmailWithLogin:(NSString *)login
                            completion:(void(^)(NSError *error))completion;

/*!
 @brief Registers a new user.
 @discussion This method tries to register a new user with specified login and password synchronously and waits for the response.
 If registration is successful, it automatically logs in the newly created user.
 @param login User's login. Cannot be nil.
 @param password User's password. Cannot be nil.
 @param doNotRecieveMarketingEmails Marketing e-mails for the user. YES for not to receive marketing letters for this account, NO for recieve.
 @param completion Block to be called upon operation completion.
 @see authorizeWithLogin:password:completion:
 */
- (void)registerWithLogin:(NSString *)login
                 password:(NSString *)password
          marketingEmails:(BOOL)doNotRecieveMarketingEmails
                partnerId:(NSString *)partnerId
               completion:(void(^)(NSError *error))completion;

- (void)loginSocialWithLogin:(NSString *)login
                 serviceType:(VPNUSocialServiceType)serviceType
                 accessToken:(NSString *)accessToken
                refreshToken:(NSString *)refreshToken // Only for Google
                     idToken:(NSString *)idToken // Only for Google
                  completion:(void(^)(NSString *longAccessToken, NSDate *expirationDate, NSError *error))completion;

- (void)registerSocialWithLogin:(NSString *)login
                    serviceType:(VPNUSocialServiceType)serviceType
                    accessToken:(NSString *)accessToken
                   refreshToken:(NSString *)refreshToken // Only for Google
                        idToken:(NSString *)idToken // Only for Google
                marketingEmails:(BOOL)doNotReceiveMarketingEmails
                      partnerId:(NSString *)partnerId
                     completion:(void(^)(NSString *longAccessToken, NSDate *expirationDate, NSError *error))completion;

- (void)confirmTermsAgreementWithMarketingEmails:(BOOL)doNotReceiveMarketingEmails
                                      completion:(void (^)(NSError *error))completion;

/*!
 @brief Signs out the already authorized user.
 @discussion This method clears current session and signals server about sign-out event.
 @param completion Block to be called upon operation completion.
 */
- (void)signOutWithCompletion:(void(^)(BOOL successful, NSError *error))completion;

/*!
 @brief Sends recovery password instructions.
 @discussion Instructions are sent to the email of user specified in login parameter.
 @param login User's login/email.
 @param completion Block to be called upon operation completion.
 Possible error responses: 361, 502, 504
 @see VPNUResponseCode
 */
- (void)recoveryPasswordForLogin:(NSString *)login completion:(void(^)(NSError *error))completion;

/*!
 @brief Retrieves a list of available personal VPN servers for currently logged-in user.
 @param squareFlags The server list is obtained with square flags
 @param completion Block to be called upon operation completion.
 */
- (void)personalServerRegionsListWithSquareFlags:(BOOL)squareFlags
                                      completion:(void(^)(NSArray<VPNUPersonalServer *> *servers, NSError *error))completion;

/*!
 @brief Retrieves a list of available personal IP servers for currently logged-in user.
 @param squareFlags The server list is obtained with square flags
 @param completion Block to be called upon operation completion.
 */
- (void)personalIPServersListWithSquareFlags:(BOOL)squareFlags completion:(void(^)(NSArray<VPNUPersonalServer *> *servers, NSError *error))completion;

/*!
 @brief Change the personal server name.
 @discussion Used to change the name of the personal server
 @param region Server region.
 @param newName New server name.
 @param completion Block to be called upon operation completion.
 */
- (void)changePersonalServerName:(NSString *)newName region:(NSString *)region completion:(void(^)(NSError *error))completion;

- (void)authorizeWithRefreshToken:(NSString *)refreshToken completion:(void (^)(NSError *))completion;

- (void)requestTwoFactorEmailCodeForUser:(NSString *)login completion:(void (^)(NSError *))completion;

@end


/*!
 @category VPN
 */

@interface KSVPNUFacade (VPN)

/*!
 @brief Starts the VPN tunnel using the current VPN configuration.
 @discussion This method returns after VPN tunnel connection process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @link status@/link property.
 @param serverItem Server item object to perform setup for. Cannot be nil.
 @param protocol Protocol for VPN configuration.
 @param completion Block to be called upon operation completion.
 */
- (void)startVPNTunnelWithServerItem:(VPNUServerItem *)serverItem
                            protocol:(VPNUProtocol)protocol
                          completion:(void(^)(NSError *error))completion;

/*!
 @brief Stops the VPN tunnel.
 @discussion This method returns after VPN tunnel disconnect process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @link connectionStatus@/link property.
 @see status
 @param completion Block to be called upon operation completion.
 */
- (void)stopVPNWithCompletion:(void(^)(NSError *error))completion;

/*!
 @brief Turns VPN On Demand on or off.
 @param enable Boolean value which controls VPN On Demand toggling. If set to YES VPN On Demand is turned on, otherwise VPN On Demand is turned off.
 @param completion Block to be called upon operation completion.
 */
- (void)enableOnDemand:(BOOL)enable completion:(void(^)(NSError *error))completion;

/*!
 @brief Turns VPN On Demand for Cellular network.
 @param enable Boolean value which controls VPN On Demand for Cellular. If set to YES VPN On Demand is turned on for all networks, otherwise VPN On Demand is turned on only for not Cellular networks (WiFi on iPhone). Default is YES.
 @param completion Block to be called upon operation completion.
 */
- (void)enableOnDemandForCellular:(BOOL)enable completion:(void (^)(NSError *error))completion;

/*!
 @brief Adds an array of the Service Set Identifier (SSID) to trusted list.
 @param list An array that contains a list of SSIDs. Cannot be nil.
 @param completion Block to be called upon operation completion.
 */
- (void)setSSIDsToTrustedList:(NSArray<NSString *> *)list completion:(void (^)(NSError *error))completion;

/*!
 @brief Removes the VPN configuration from the Network Extension preferences if present.
 @param completion Block to be called upon operation completion.
 */
- (void)removeProfileWithCompletion:(void(^)(NSError *error))completion;

@end


/*!
 @category FakeLogin
 */

@interface KSVPNUFacade (FakeLogin)

/*!
 @brief Automatic register a new user.
 @discussion This method tries to register a new user with automatically generated login and password, based on companyDomain synchronously and waits for the response.
 If registration is successful, it automatically logs in the newly created user.
 @param companyDomain Your company domain e.g. 'mycompany.com'. Cannot be nil.
 @param completion Block to be called upon operation completion.
 */
- (void)registerFakeAccountWithCompanyDomain:(NSString *)companyDomain
                                   partnerId:(NSString *)partnerId
                                  completion:(void(^)(NSString * login ,NSString * password, NSError * error))completion;

/*!
 @brief Creates a new account from the automatically generated credentials.
 @discussion This method tries to create a new account with real credentials and merge old fake account to newly created. All purchases from fake account will be transfered to new account. If merge is successful, it automatically logs in the newly created user.
 @param login User's login. Cannot be nil.
 @param password User's password. Cannot be nil.
 @param fakeLogin Fake login. Cannot be nil.
 @param fakePassword Fake password. Cannot be nil.
 @param doNotReceiveMarketingEmails Marketing e-mails for the user. YES for not to receive marketing letters for this account, NO for recieve.
 @param completion Block to be called upon operation completion.
 */
- (void)mergeLogin:(NSString *)login
     withFakeLogin:(NSString *)fakeLogin
      fakePassword:(NSString *)fakePassword
          password:(NSString *)password
   marketingEmails:(BOOL)doNotReceiveMarketingEmails
        completion:(void(^)(NSError * error))completion;

/*!
 @brief Save account credentials in users iCloud using iCloud Key-Value Storage.
 @discussion If the user is authorized in iCloud on the device, the method will try to save the data using iCloud Key-Value Storage. It should be used in conjunction with Fake Account
 @param login User's login. Cannot be nil.
 @param password User's password. Cannot be nil.
 */
- (void)saveAccountInCloudLogin:(NSString *)login password:(NSString *)password;

/*!
 @brief Load account credentials from user iCloud using iCloud Key-Value Storage.
 @discussion If the user is authorized in iCloud on the device, the method will try to load the data using iCloud Key-Value Storage. It should be used in conjunction with Fake Account
 */
- (NSDictionary *)loadAccountFromCloud;

/*!
 @brief Generated account for current device.
 */
- (NSDictionary *)accountForCurrentDeviceWithCompanyDomain:(NSString *)companyDomain;

@end

#if TARGET_OS_OSX
/*!
 @category Standalone
 */

@interface KSVPNUFacade (Standalone)

// Helper Tool
@property (readonly) BOOL isHelperToolInstalled;

- (void)installHelperToolWithCompletion:(void(^)(NSError *error))completion;
- (void)removeHelperTool:(void(^)(void))completion;
- (void)helperToolVersionWithCompletion:(void (^)(NSString *version))completion;


// Kill Switch
- (void)startKillSwitchWithDomain:(NSString *)domain
                   userAuthorized:(BOOL)authorized
                       completion:(void(^)(NSError *error))completion;

- (void)stopKillSwitchWithCompletion:(void(^)(void))completion;

@end

#endif

@interface KSVPNUFacade (CrossAuth)

@property (readonly) VPNUCrossAuthData *currentAppAuthData;

@property (readonly) NSArray<VPNUCrossAuthData *> *otherCompanyAppsAuthData;

- (void)removeCrossAuthDataForCurrentApp;

@end





