//
//  VPNUStorage.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

/*!
 @header
 @abstract Header file
 */

#import <Foundation/Foundation.h>

@class VPNUAccountUserInfo;


/*!
 @class VPNUStorage
 @abstract
 Special class which can be used to save account-specific information on disk.
 @discussion Use this class if you want to save any information which is specific for an account described by @link //apple_ref/occ/cl/VPNUAccountUserInfo@/link. This class also provides an interface to save data securely using methods which use encryption and decryption.
 
 The class uses NSUserDefaults internally. If you clear the defaults associated with the intance of this class (generally standardUserDefaults), all stored values (including encrypted) will be lost.
 */
@interface VPNUStorage : NSObject

/*!
 @brief Encrypts and stores provided key-value pair (key and object) for the specified account.
 @discussion This method generates an exception if no encryption key was provided using @link setEncryptionKey:@/link method.
 The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store. Cannot be nil.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key-value pair.
 */
- (void)storeWithEncryptionObject:(id)object forKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account;
/*!
 @brief Decrypts and returns the object associated with the provided key and account.
 @discussion This method generates an exception if no encryption key was provided using @link setEncryptionKey:@/link method.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key.
 */
- (id)readWithDecryptionObjectForKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account;

/*!
 @brief Stores provided key-value pair (key and object) for the specified account.
 @discussion The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key-value pair.
 */
- (void)storeObject:(id)object forKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account;
/*!
 @brief Returns the object associated with the provided key and account.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key.
 */
- (id)readObjectForKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account;


/*!
 @brief Encrypts and stores provided object for the specified key.
 @discussion This method generates an exception if no encryption key was provided using @link setEncryptionKey:@/link method.
 The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store. Cannot be nil.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (void)storeWithEncryptionObject:(id)object forKey:(NSString *)key;
/*!
 @brief Decrypts and returns the object associated with the provided key.
 @discussion This method generates an exception if no encryption key was provided using @link setEncryptionKey:@/link method.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (id)readWithDecryptionObjectForKey:(NSString *)key;
/*!
 @brief Stores provided object for the specified key.
 @discussion The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (void)storeObject:(id)object forKey:(NSString *)key;
/*!
 @brief Returns the object associated with the provided key.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (id)readObjectForKey:(NSString *)key;


/*!
 @brief Sets the provided string as the encryption and decryption key used by secure methods.
 @discussion The provided string is later used internally when the methods with pattern @link storeWithEncryptionObject:forKey:@/link and @link readWithDecryptionObjectForKey:@/link are called. It is recommended to use an arbitrary, comparatively large and permutated string for this value. Generally you are discouraged to use a user's login or password or any meaningful string as an encryption key.
 
 Set the key only once before making any calls to the instance. Otherwise the correctness of encryption and decryption is not guaranteed.
 
 Encryption key should be the same between the launches of your host application. If you change the key, all previously saved values will be inaccessible until you set the key back to the previous value.
 
 @param key Encryption key to be used for encryption. Cannot be nil.
 */
- (void)setEncryptionKey:(NSString *)key;

/*!
 @brief Returns the shared storage object.
 @discussion The shared storage is created internally with standardUserDefaults instance of NSUserDefaults class. If you want to use any custom NSUserDefaults object, you can initialize your own instance of @link VPNUStorage@/link using @link initWithUserDefaults:@/link covinience initializer.
 */
+ (instancetype)defaultStorage;
/*!
 @brief Initializes and return a newly created @link VPNUStorage@/link instance which uses a custom NSUserDefaults object.
 @discussion This method is a convenience constructor.
 
 Use this method only if you know what you are doing. Generally you want to use standardUserDefaults instance of NSUserDefaults class - simply use the defaultStorage class method (the latter is generally preferred).
 
 @param userDefaults A custom NSUserDefaults object.
 */
- (instancetype)initWithUserDefaults:(NSUserDefaults *)userDefaults NS_DESIGNATED_INITIALIZER;

@end
